@@ -7,6 +7,7 @@ import (
 	"io/ioutil"
 	"log"
 	"net/http"
+	"net/url"
 	"os"
 	"os/exec"
 	"os/user"
@@ -133,21 +134,26 @@ func (u *upstream) get(gopath, repo, rev string) error {
 	return rr.VCS.Create(dir, rr.Repo)
 }
 
-func (u *upstream) tarballFromHoster(repo string) error {
-	var url string
-	parts := strings.Split(repo, "/")
-	if len(parts) < 3 {
-		return fmt.Errorf("Unsupported hoster")
+func (u *upstream) tarballFromHoster() error {
+	var tarURL string
+	repo := strings.TrimSuffix(u.rr.Repo, ".git")
+	repoU, err := url.Parse(repo)
+	if err != nil {
+		return err
 	}
-	host, owner, project := parts[0], parts[1], parts[2]
 
-	switch host {
+	switch repoU.Host {
 	case "github.com":
-		url = fmt.Sprintf("https://%s/%s/%s/archive/v%s.tar.%s",
-			host, owner, project, u.version, u.compression)
+		tarURL = fmt.Sprintf("%s/archive/v%s.tar.%s",
+			repo, u.version, u.compression)
 	case "gitlab.com":
-		url = fmt.Sprintf("https://%s/%s/%s/-/archive/v%s/%s-%s.tar.%s",
-			host, owner, project, u.version, project, u.version, u.compression)
+		parts := strings.Split(repoU.Path, "/")
+		if len(parts) < 3 {
+			return fmt.Errorf("Incomplete repo URL: %s", u.rr.Repo)
+		}
+		project := parts[2]
+		tarURL = fmt.Sprintf("%s/-/archive/v%[3]s/%[2]s-%s.tar.%s",
+			repo, project, u.version, u.compression)
 	default:
 		return fmt.Errorf("Unsupported hoster")
 	}
@@ -155,8 +161,8 @@ func (u *upstream) tarballFromHoster(repo string) error {
 	done := make(chan struct{})
 	go progressSize("Download", u.tarPath, done)
 
-	log.Printf("Downloading %s", url)
-	err := downloadFile(u.tarPath, url)
+	log.Printf("Downloading %s", tarURL)
+	err = downloadFile(u.tarPath, tarURL)
 
 	close(done)
 
@@ -176,7 +182,7 @@ func (u *upstream) tar(gopath, repo string) error {
 			log.Printf("Godeps/_workspace exists, not downloading tarball from hoster.")
 		} else {
 			u.compression = "gz"
-			err := u.tarballFromHoster(repo)
+			err := u.tarballFromHoster()
 			if err != nil && err.Error() == "Unsupported hoster" {
 				log.Printf("INFO: Hoster does not provide release tarball\n")
 			} else {
@@ -410,6 +416,14 @@ func createGitRepository(debsrc, gopkg, orig string, u *upstream,
 	if err := runGitCommandIn(dir, "config", "push.default", "matching"); err != nil {
 		return dir, err
 	}
+
+	// [remote "origin"]
+
+	originURL := "git@salsa.debian.org:go-team/packages/" + debsrc + ".git"
+	log.Printf("Adding remote \"origin\" with URL %q\n", originURL)
+	if err := runGitCommandIn(dir, "remote", "add", "origin", originURL); err != nil {
+		return dir, err
+	}
 	if err := runGitCommandIn(dir, "config", "--add", "remote.origin.push", "+refs/heads/*:refs/heads/*"); err != nil {
 		return dir, err
 	}
@@ -417,12 +431,33 @@ func createGitRepository(debsrc, gopkg, orig string, u *upstream,
 		return dir, err
 	}
 
+	// Preconfigure branches
+
+	var debianBranch string
+	if dep14 {
+		debianBranch = "debian/sid"
+	} else {
+		debianBranch = "master"
+	}
+	branches := []string{debianBranch, "upstream"}
+	if pristineTar {
+		branches = append(branches, "pristine-tar")
+	}
+	for _, branch := range branches {
+		if err := runGitCommandIn(dir, "config", "branch."+branch+".remote", "origin"); err != nil {
+			return dir, err
+		}
+		if err := runGitCommandIn(dir, "config", "branch."+branch+".merge", "refs/heads/"+branch); err != nil {
+			return dir, err
+		}
+	}
+
 	if includeUpstreamHistory {
 		u.remote, err = shortHostName(gopkg, allowUnknownHoster)
 		if err != nil {
 			return dir, fmt.Errorf("Unable to fetch upstream history: %q", err)
 		}
-		log.Printf("Adding %q as remote %q\n", u.rr.Repo, u.remote)
+		log.Printf("Adding remote %q with URL %q\n", u.remote, u.rr.Repo)
 		if err := runGitCommandIn(dir, "remote", "add", u.remote, u.rr.Repo); err != nil {
 			return dir, err
 		}
@@ -508,48 +543,41 @@ func normalizeDebianProgramName(str string) string {
 }
 
 func shortHostName(gopkg string, allowUnknownHoster bool) (host string, err error) {
+	knownHosts := map[string]string{
+		// keep the list in alphabetical order
+		"bazil.org":         "bazil",
+		"bitbucket.org":     "bitbucket",
+		"blitiri.com.ar":    "blitiri",
+		"cloud.google.com":  "googlecloud",
+		"code.google.com":   "googlecode",
+		"git.sr.ht":         "sourcehut",
+		"github.com":        "github",
+		"gitlab.com":        "gitlab",
+		"go.uber.org":       "uber",
+		"go4.org":           "go4",
+		"gocloud.dev":       "gocloud",
+		"golang.org":        "golang",
+		"google.golang.org": "google",
+		"gopkg.in":          "gopkg",
+		"honnef.co":         "honnef",
+		"howett.net":        "howett",
+		"k8s.io":            "k8s",
+		"pault.ag":          "pault",
+		"salsa.debian.org":  "debian",
+		"sigs.k8s.io":       "k8s-sigs",
+	}
 	parts := strings.Split(gopkg, "/")
 	fqdn := parts[0]
-
-	switch fqdn {
-	case "github.com":
-		host = "github"
-	case "code.google.com":
-		host = "googlecode"
-	case "cloud.google.com":
-		host = "googlecloud"
-	case "gopkg.in":
-		host = "gopkg"
-	case "golang.org":
-		host = "golang"
-	case "google.golang.org":
-		host = "google"
-	case "gitlab.com":
-		host = "gitlab"
-	case "bitbucket.org":
-		host = "bitbucket"
-	case "bazil.org":
-		host = "bazil"
-	case "blitiri.com.ar":
-		host = "blitiri"
-	case "pault.ag":
-		host = "pault"
-	case "howett.net":
-		host = "howett"
-	case "go4.org":
-		host = "go4"
-	case "salsa.debian.org":
-		host = "debian"
-	default:
-		if allowUnknownHoster {
-			suffix, _ := publicsuffix.PublicSuffix(host)
-			host = host[:len(host)-len(suffix)-len(".")]
-			log.Printf("WARNING: Using %q as canonical hostname for %q. If that is not okay, please file a bug against %s.\n", host, fqdn, os.Args[0])
-		} else {
-			err = fmt.Errorf("unknown hoster %q", fqdn)
-		}
+	if host, ok := knownHosts[fqdn]; ok {
+		return host, nil
 	}
-	return host, err
+	if !allowUnknownHoster {
+		return "", fmt.Errorf("unknown hoster %q", fqdn)
+	}
+	suffix, _ := publicsuffix.PublicSuffix(fqdn)
+	host = fqdn[:len(fqdn)-len(suffix)-len(".")]
+	log.Printf("WARNING: Using %q as canonical hostname for %q. If that is not okay, please file a bug against %s.\n", host, fqdn, os.Args[0])
+	return host, nil
 }
 
 // debianNameFromGopkg maps a Go package repo path to a Debian package name,
@@ -756,6 +784,14 @@ func execMake(args []string, usage func()) {
 			"Valid values are \"a\", \"at\" and \"ast\", see wrap-and-sort(1) man page\n"+
 			"for more information.")
 
+	// ====================================================================
+	//
+	// Start actual make routine
+	//
+	// ====================================================================
+
+	log.Printf("Starting %q", buildVersionString())
+
 	err := fs.Parse(args)
 	if err != nil {
 		log.Fatal(err)
@@ -940,7 +976,7 @@ func execMake(args []string, usage func()) {
 	}
 	fmt.Printf("\n")
 	fmt.Printf("Resolve all TODOs in %s, then email it out:\n", itpname)
-	fmt.Printf("    sendmail -t < %s\n", itpname)
+	fmt.Printf("    /usr/sbin/sendmail -t < %s\n", itpname)
 	fmt.Printf("\n")
 	fmt.Printf("Resolve all the TODOs in debian/, find them using:\n")
 	fmt.Printf("    grep -r TODO debian\n")
@@ -953,7 +989,6 @@ func execMake(args []string, usage func()) {
 	fmt.Printf("    dh-make-golang create-salsa-project %s\n", debsrc)
 	fmt.Printf("\n")
 	fmt.Printf("Once you are happy with your packaging, push it to salsa using:\n")
-	fmt.Printf("    git remote set-url origin git@salsa.debian.org:go-team/packages/%s.git\n", debsrc)
 	fmt.Printf("    gbp push\n")
 	fmt.Printf("\n")
 
@@ -961,7 +996,7 @@ func execMake(args []string, usage func()) {
 		fmt.Printf("NOTE: Full upstream git history has been included as per pkg-go team's\n")
 		fmt.Printf("      new workflow.  This feature is new and somewhat experimental,\n")
 		fmt.Printf("      and all feedback are welcome!\n")
-		fmt.Printf("      (For old behavior, use --include-upstream-history=false)\n")
+		fmt.Printf("      (For old behavior, use --upstream-git-history=false)\n")
 		fmt.Printf("\n")
 		fmt.Printf("The upstream git history is being tracked with the remote named %q.\n", u.remote)
 		fmt.Printf("To upgrade to the latest upstream version, you may use something like:\n")
